#!/usr/bin/env python
"""
This simple example is used for the line-by-line tutorial
that comes with pygame. It is based on a 'popular' web banner.
Note there are comments here, but for the full explanation, 
follow along in the tutorial.
"""


#Import Modules
import os, pygame, operator, math
from copy import copy
from pygame.locals import *
from random import Random

if not pygame.font: print 'Warning, fonts disabled'
if not pygame.mixer: print 'Warning, sound disabled'

RESOLUTION = (900,900)
vectorSum = lambda a,b: (a[0]+b[0], a[1]+b[1])
RED    = (255,  0,    0)
BLACK  = (0,    0,    0)
WHITE  = (255,  255,  255)


rng = Random()

#-----------------------------------------------------------------------------
def load_png(name):
    fullname = os.path.join('data', name)
    try:
        image = pygame.image.load(fullname)
	if image.get_alpha is None:
		print "no alpha"
		image = image.convert()
	else:
		image = image.convert_alpha()
    except pygame.error, message:
        print 'Cannot load image:', fullname
        raise SystemExit, message
    return image, image.get_rect()

#-----------------------------------------------------------------------------
def load_sound(name):
    class NoneSound:
        def play(self): pass
    if not pygame.mixer or not pygame.mixer.get_init():
        return NoneSound()
    fullname = os.path.join('data', name)
    try:
        sound = pygame.mixer.Sound(fullname)
    except pygame.error, message:
        print 'Cannot load sound:', fullname
        raise SystemExit, message
    return sound



#-----------------------------------------------------------------------------
class SimpleSprite(pygame.sprite.Sprite):
	def __init__(self, imgName=None, center=(0,0) ):
		pygame.sprite.Sprite.__init__(self) #call Sprite initializer
		if imgName:
			self.image, self.rect = load_png(imgName)
		else:
			self.image = pygame.Surface( (100,100) )
			self.image.fill( (200,0,0) )
			self.rect = self.image.get_rect()

		self.rect.center = center
	#---------------------------------------------------------------------
	def NotifyDirtyScreen(self):
		pass
	#---------------------------------------------------------------------
	def update(self):
		pass
	#---------------------------------------------------------------------
	def SetImg(self, newSurface, newRect=None):
		self.image = newSurface
		if newRect:
			self.rect  = newRect
		else:
			self.rect  = newSurface.get_rect()


#classes for our game objects
#-----------------------------------------------------------------------------
class GameLevel:
	def __init__( self ):
		self.paused = 0

		self.secondsToChange = 10
		self.secondsCounter = 0

		self.soundQueue = []
		self.bipSound = load_sound( 'bip.ogg' )

		self.bottomGroup = pygame.sprite.RenderUpdates()
		self.topGroup     = pygame.sprite.RenderUpdates()

		self.background = pygame.Surface( RESOLUTION )
		self.background.fill( WHITE )

		center = screen.get_rect().center

		self.pauseSprite = SimpleSprite( 'paused.png', center )
		self.pauseSprite.rect.move_ip( 0, 300 )

		self.circles = {
		  'yellow': 
		           {'img': SimpleSprite( 'circle_yellow.png', center ),
		            'str': "YELLOW",
		            'snd': load_sound( 'circle_yellow.ogg' )
		           },
		  'green' : 
		           {'img': SimpleSprite( 'circle_green.png', center ),
		            'str': "GREEN",
		            'snd': load_sound( 'circle_green.ogg' )
		           },
		  'blue'  : 
		           {'img': SimpleSprite( 'circle_blue.png', center ),
		            'str': "BLUE",
		            'snd': load_sound( 'circle_blue.ogg' )
		           },
		  'red'   : 
		           {'img': SimpleSprite( 'circle_red.png', center ),
		            'str': "RED",
		            'snd': load_sound( 'circle_red.ogg' )
		           },
		}
		self.bodyParts = {
		  'left hand' : 
		           {'img': SimpleSprite( 'hand_l.png', center ),
		            'str': "LEFT HAND",
		            'snd': load_sound( 'hand_l.ogg' )
		           },
		  'right hand': 
		           {'img': SimpleSprite( 'hand_r.png', center ),
		            'str': "RIGHT HAND",
		            'snd': load_sound( 'hand_r.ogg' )
		           },
		  'left foot' : 
		           {'img': SimpleSprite( 'foot_l.png', center ),
		            'str': "LEFT FOOT",
		            'snd': load_sound( 'foot_l.ogg' )
		           },
		  'right foot': 
		           {'img': SimpleSprite( 'foot_r.png', center ),
		            'str': "RIGHT FOOT",
		            'snd': load_sound( 'foot_r.ogg' )
		           },
		}

		self.RandomChoice()

		from statusbar import StatusBar
		hud = StatusBar( self, (20,20,200,50), 
		                 attrName='secondsCounter',
		                 fullAmt=9 )
		self.topGroup.add( hud )


	
	#---------------------------------------------------------------------
	def Signal( self, event ):
		if event.type == KEYDOWN:
			if event.key == K_SPACE:
				self.Pause()
			else:
				self.Change()
		elif event.type == USEREVENT:
			self.Change()

	#---------------------------------------------------------------------
	def Pause( self ):
		self.paused = (not self.paused)

		if self.paused:
			self.topGroup.add( self.pauseSprite )
		else:
			self.topGroup.remove( self.pauseSprite )

	#---------------------------------------------------------------------
	def Change( self ):
		if self.paused:
			return

		if len( self.soundQueue ) > 0:
			s = self.soundQueue.pop(0)
			s.play()
		else:
			self.bipSound.play()

		self.secondsCounter += 1
		if self.secondsCounter < self.secondsToChange:
			return
		self.secondsCounter = 0

		self.topGroup.remove( 
		                  self.bodyParts[self.currentBodyPart]['img'] )
		self.bottomGroup.remove( 
		                  self.circles[self.currentCircle]['img'] )
		self.RandomChoice()

	#---------------------------------------------------------------------
	def RandomChoice( self ):

		self.currentBodyPart = rng.choice( self.bodyParts.keys() )

		self.topGroup.add( self.bodyParts[self.currentBodyPart]['img'] )

		self.currentCircle = rng.choice( self.circles.keys() )

		self.bottomGroup.add( self.circles[self.currentCircle]['img'] )

		self.bodyParts[self.currentBodyPart]['snd'].play()
		self.soundQueue.append(self.circles[self.currentCircle]['snd'])
		
	#---------------------------------------------------------------------
	def DoGraphics( self, screen, display ):
		self.bottomGroup.clear( screen, self.background )
		self.topGroup.clear( screen , self.background )

		self.bottomGroup.update()
		self.topGroup.update()

		changedRects = self.bottomGroup.draw(screen)
		changedRects += self.topGroup.draw(screen)
		display.update( changedRects )

		



	
#-----------------------------------------------------------------------------
def main():
	"""this function is called when the program starts.
	   it initializes everything it needs, then runs in
	   a loop until the function returns."""
	#Initialize Everything
	global screen
	screen = None
	pygame.init()
	screen = pygame.display.set_mode(RESOLUTION)
	pygame.display.set_caption('Twister Referee')
	#pygame.mouse.set_visible(0)

	#Display The Background
	screen.fill( WHITE )
	pygame.display.flip()

	#Prepare Game Objects
	clock = pygame.time.Clock()

	twister = GameLevel()

	#Main Loop
	tickSpeed = 40
	counter = 0
	oldfps = 0
	while 1:
		clock.tick(tickSpeed)
		counter += 1

		newfps = int(clock.get_fps())
		if newfps is not oldfps:
			print "fps: ", clock.get_fps()
			oldfps = newfps


		if counter >= tickSpeed:
			counter = 0
			ev = pygame.event.Event( USEREVENT, {'tick':1} )
			pygame.event.post( ev )

		#Handle Input Events
		remainingEvents = pygame.event.get()

		if len( remainingEvents ) == 0:
			continue;

		for event in remainingEvents:
			if event.type == QUIT:
				return
			elif event.type == KEYDOWN and event.key == K_ESCAPE:
				return
			else:
				twister.Signal( event )
			#elif event.type == KEYDOWN or event.type == KEYUP:
				#twister.SignalKey( event )

		#Draw Everything

		twister.DoGraphics( screen, pygame.display )


	#Game Over
	pygame.quit()


#this calls the 'main' function when this script is executed
if __name__ == '__main__': main()
